package fr.ensicaen.ecole.easygame;

import java.awt.Dimension;
import java.awt.Image;
import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

import fr.ensicaen.ecole.easygame.gui.GamePane;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.StartGameMessage;
import fr.ensicaen.ecole.easygame.plugin.Plugin;
import fr.ensicaen.ecole.easygame.state.GameState;
import fr.ensicaen.ecole.easygame.utils.ObjectUtils;

/**
 * A game implemented using the framework.
 */
public final class Game {

	private GameState _gameState;
	private GameState _turnStartGameState;
	private GamePane _pane;
	private boolean _turnHasBegan = false;
	private boolean _isEndOfTurn;
	private boolean _canSubmitTurn = false;
	private List<Plugin> _plugins = new ArrayList<Plugin>();
	private int _localPlayerId = -1;
	private boolean _canCancelTurn = false;
	private Dimension _inventoryEntitySize = null;
	private boolean _isWindowClosed = false;
	private ClassLoader _loader;
	private Image _image;

	/**
	 * Constructs a game using specific information from the game manager.
	 *
	 * @param lgame Main class of the game.
	 * @param makeUI Set to true if a GUI needs to be created.
	 */
	public Game(LaunchableGame lgame, boolean makeUI, ClassLoader loader) {
		_canCancelTurn = lgame.isCancelTurnAllowed();
		_inventoryEntitySize = lgame.getInventoryEntitySize();
		_loader = loader;
		_image = lgame.getImage();

		if (makeUI)
			_pane = new GamePane(this);
	}

	/**
	 * Returns the current game state.
	 *
	 * @return Current game state.
	 */
	public GameState getGameState() {
		return _gameState;
	}

	/**
	 * Modifies the current game state.
	 * This also refreshes the window.
	 *
	 * @param gameState The new game state.
	 */
	public void setGameState(GameState gameState) {
		_gameState = gameState;
		_isEndOfTurn = false;

		if (!_turnHasBegan && _gameState.getCurrentPlayer() != null
				&& _localPlayerId == _gameState.getCurrentPlayer().getId()) {
			_turnHasBegan = true;
			_canSubmitTurn = false;
			_turnStartGameState = (GameState) ObjectUtils.deepCopy(_gameState, _loader);
		}

		if (_pane != null) {
			_pane.updatePlayerCount();
			_pane.getInventoryZone().updateInventoryView();
			_pane.getStatusColumn().setEndOfTurnEnabled(false);
			_pane.repaint();
		}
	}

	/**
	 * Resets the turn.
	 */
	public void resetTurn() {
		if (_gameState.getCurrentPlayer().getId() == _localPlayerId)
			setGameState((GameState) ObjectUtils.deepCopy(_turnStartGameState, _loader));
	}

	/**
	 * Used to start a game.
	 */
	public void run() {
		MessageReceiver.receive(new StartGameMessage());
		nextTurn();
	}

	/**
	 * Ends the current turn.
	 */
	public void endTurn() {
		_isEndOfTurn = true;

		if (_pane != null) {
			_pane.getStatusColumn().setEndOfTurnEnabled(true);
		}

		if (!_canCancelTurn) {
			submitTurn();
		}
	}

	/**
	 * Sends the game state to the server if the turn has ended.
	 */
	public void submitTurn() {
		if (!isEndOfTurn()) {
			return;
		}

		_canSubmitTurn = true;
		_turnHasBegan = false;

		synchronized (this) {
			notifyAll();
		}
	}

	/**
	 * @return Whether or not the current turn has ended.
	 */
	public boolean isEndOfTurn() {
		return _isEndOfTurn;
	}

	void nextTurn() {
		_gameState.nextTurn();
	}

	/**
	 * Ends the game.
	 */
	public void endGame() {
		_gameState.setEndOfGame();
	}

	/**
	 * @return The GUI in which the game is displayed.
	 */
	public GamePane getPane() {
		return _pane;
	}

	/**
	 * Triggers a complete repaint of the game window.
	 */
	public void repaint() {
		if (_pane != null) {
			_pane.repaint();
		}
	}

	/**
	 * Adds a plugin the the game.
	 *
	 * @param plugin The plugin to be added.
	 */
	public void addPlugin(Plugin plugin) {
		_plugins.add(plugin);
	}

	/**
	 * @return Number of plugins added to the game.
	 */
	public int getNbPlugins() {
		return _plugins.size();
	}

	/**
	 * @param i Index of the plugin to return.
	 * @return The plugin at the index i.
	 */
	public Plugin getPlugin(int i) {
		return _plugins.get(i);
	}

	/**
	 * @return Whether the game has ended or not.
	 */
	public boolean isEndOfGame() {
		return _gameState.isEndOfGame();
	}

	/**
	 * Sets the ID of the local player. Used only by the game manager.
	 *
	 * @param id ID of the local player.
	 */
	public void setLocalPlayerId(int id) {
		this._localPlayerId = id;
	}

	/**
	 * @return The ID of the local player (i.e. the one playing in the current window).
	 */
	public int getLocalPlayerId() {
		return this._localPlayerId;
	}

	/**
	 * @return Whether the turn is ready to be submitted to the server or not.
	 */
	public boolean canSubmitTurn() {
		return _canSubmitTurn;
	}

	/**
	 * @return Whether turns can be canceled or not.
	 */
	public boolean canCancelTurn() {
		return _canCancelTurn;
	}

	/**
	 * @return The size of an entity to be displayed in the inventory.
	 */
	public Dimension getInventoryEntitySize() {
		return _inventoryEntitySize;
	}

	/**
	 * Closes the window.
	 */
	public void closeWindow() {
		SwingUtilities.getWindowAncestor(_pane).dispose();
		_isWindowClosed = true;
		synchronized (this) {
			notifyAll();
		}
	}

	/**
	 * @return Whether the window has been closed or not.
	 */
	public boolean isWindowClosed() {
		return _isWindowClosed;
	}

	public Image getImage() {
		return _image;
	}
}
