package fr.ensicaen.ecole.easygame;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import fr.ensicaen.ecole.easygame.events.MouseClickEvent;
import fr.ensicaen.ecole.easygame.messaging.AfterTurnMessage;
import fr.ensicaen.ecole.easygame.messaging.AfterTurnSubmitMessage;
import fr.ensicaen.ecole.easygame.messaging.BeforeTurnMessage;
import fr.ensicaen.ecole.easygame.messaging.EndGameMessage;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.MouseMessage;
import fr.ensicaen.ecole.easygame.state.GameState;

public final class HumanClient extends Thread implements Client {

	private Socket _socket;
	private final Game _game;
	private MessageReceiver _logic;
	private JFrame _window;
	private Queue<MouseClickEvent> _events = new ConcurrentLinkedQueue<>();
	private boolean _shared;
	private String _address;
	private int _port;
	private String _wantedNickname;
	private ClassLoader _loader;

	public HumanClient(String address, int port, LaunchableGame lgame, JFrame window, ClassLoader loader, String wantedNickname) {
		_game = new Game(lgame, true, loader);
		_logic = lgame.instantiateLogic();
		_window = window;
		_address = address;
		_port = port;
		_wantedNickname = wantedNickname;
		_loader = loader;

		_game.getPane().getGameZone().addMouseListener(new MouseListener() {
			@Override
			public void mouseClicked(MouseEvent e) {
				int x = e.getX() - _game.getPane().getGameZone().getOffsetX();
				int y = e.getY() - _game.getPane().getGameZone().getOffsetY();

				_events.add(new MouseClickEvent(x, y));

				synchronized (_game) {
					_game.notify();
				}
			}

			@Override
			public void mousePressed(MouseEvent e) {
			}

			@Override
			public void mouseReleased(MouseEvent e) {
			}

			@Override
			public void mouseEntered(MouseEvent e) {
			}

			@Override
			public void mouseExited(MouseEvent e) {
			}
		});
	}

	public void setShared() {
		this._shared = true;
	}

	public boolean tryConnect() {
		try {
			_socket = new Socket(_address, _port);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public void run() {
		boolean isInitialized = false;
		boolean isPersonalInitialized = false;

		_logic.link(_game);

		try {
			ObjectInputStream streamInput;

			if (_loader != null) {
				streamInput = new SmarterObjectInputStream(_socket.getInputStream(), _loader);
			} else {
				streamInput = new ObjectInputStream(_socket.getInputStream());
			}

			ObjectOutputStream streamOutput = new ObjectOutputStream(_socket.getOutputStream());

			streamOutput.writeObject(_wantedNickname);
			int id = (Integer) streamInput.readObject();
			_game.setLocalPlayerId(id);

			while (true) {
				GameState gs = (GameState) streamInput.readObject();
				_game.setGameState(gs);

				if (_game.isEndOfGame()) {
					break;
				}

				if (!_shared || id == _game.getGameState().getCurrentPlayer().getId()) {
					JPanel panel = (JPanel) _window.getContentPane();
					panel.removeAll();
					panel.add(_game.getPane());

					if (isInitialized) {
						_window.setPreferredSize(_window.getSize());
					}

					_window.pack();
					_game.getPane().repaint();
				}

				if (!isInitialized) {
					isInitialized = true;
					_game.getPane().postInitBuild();
					_window.pack();
				}

				if (!isPersonalInitialized && id == _game.getGameState().getCurrentPlayer().getId()) {
					isPersonalInitialized = true;
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							_window.pack();
							_game.getPane().centerView();
						}
					});
				}

				if (id == _game.getGameState().getCurrentPlayer().getId()) {
					MessageReceiver.receive(new BeforeTurnMessage());
					_events.clear();

					while (!_game.canSubmitTurn() && !_game.isWindowClosed()) {
						try {
							synchronized (_game) {
								_game.wait();
							}
						} catch (InterruptedException e) {
							e.printStackTrace();
						}

						while (!_events.isEmpty()) {
							MessageReceiver.receive(new MouseMessage(_events.poll()));
						}
					}

					if (_game.isWindowClosed()) {
						break;
					}

					MessageReceiver.receive(new AfterTurnSubmitMessage());
					_game.nextTurn();
					streamOutput.writeObject(_game.getGameState());
				}
			}

			if ((!_shared || id == _game.getGameState().getCurrentPlayer().getId())
					&& !_game.isWindowClosed()) {
				MessageReceiver.receive(new EndGameMessage(_window));
			}

			streamInput.close();
			streamOutput.close();
		} catch (IOException e) {
			e.printStackTrace();
			JDialog errorDialog = new JDialog();
			errorDialog.setSize(350, 100);
			errorDialog.setTitle("Erreur connexion");
			errorDialog.add(new JLabel("Erreur de communication avec un client, fin du jeu."));
			errorDialog.setVisible(true);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void registerPlugins(LaunchableGame lg) {
		lg.registerPlugins(_game);
	}
}
