package fr.ensicaen.ecole.easygame;

import java.awt.Dimension;
import java.awt.Image;
import java.util.ArrayList;
import java.util.List;

import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.state.GameState;

/**
 * Implemented by games that can be launched.
 */
public abstract class LaunchableGame {

	/**
	 * @return The list of classes available for AI creation.
	 */
	public final List<Class<? extends AILogic>> getAvailableAIs() {
		List<Class<? extends AILogic>> list = new ArrayList<>();
		collectAIClasses(list);
		return list;
	}

	/**
	 * @return Returns the object responsible for handling messages and game logic.
	 */
	public abstract MessageReceiver instantiateLogic();

	/**
	 * @return Returns the initial game state for the game.
	 */
	public abstract GameState instantiateGameState();

	/**
	 * Adds all necessary plugins to the game.
	 *
	 * @param game The game where plugins have to register to.
	 */
	public abstract void registerPlugins(Game game);

	/**
	 * Collects all AI classes in the list given as argument.
	 *
	 * @param l List containing the AI classes.
	 */
	protected abstract void collectAIClasses(List<Class<? extends AILogic>> l);

	/**
	 * Informs about the size of the entities in the inventory.
	 *
	 * @return Size of an entity in the inventory.
	 */
	public abstract Dimension getInventoryEntitySize();

	/**
	 * Informs about whether or not canceling a turn is possible.
	 * If not, Game.endTurn() will end the turn.
	 * If it is allowed, the turn will have to be ended with Game.endTurn() and then Game.submitTurn() will have
	 * to be called to actually send the new state to the server. The player clicking the "End turn" button will
	 * have the same effect, but then player will also have the choice of canceling his turning and going back to
	 * state of the game when his turned began.
	 *
	 * @return Whether the player is allowed to cancel his turn or not.
	 */
	public abstract boolean isCancelTurnAllowed();

	/**
	 * @return The image icon for this game.
	 */
	public abstract Image getImage();
}
