package fr.ensicaen.ecole.easygame.messaging;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import fr.ensicaen.ecole.easygame.Game;

/**
 * Implements an object capable of handling messages.
 * For example, to receive an message called EndGameMessage,
 * the class has to implement a function "onEndGame" or "endGame" taking as parameters
 * the game and if necessary the message.
 */
public abstract class MessageReceiver {

	private static Map<Thread, MessageReceiver> _receivers = new HashMap<>();

	private Game _game;
	private Map<Class<?>, Method> _methodCache = new HashMap<>();

	private static void register(MessageReceiver recv) {
		// TODO: check duplicates
		_receivers.put(Thread.currentThread(), recv);
	}

	/**
	 * Dispatches a message.
	 *
	 * @param m Message to be dispatched.
	 */
	public static void receive(Message m) {
		Thread th = Thread.currentThread();

		if (!_receivers.containsKey(th)) {
			throw new RuntimeException(th + " has no receivers to receive " + m);
		}

		_receivers.get(th).proceed(m);
	}

	public final void link(Game game) {
		register(this);
		_game = game;
	}

	private void proceed(Message m) {
		if (!_methodCache.containsKey(m.getClass())) {
			String className = m.getClass().getSimpleName().replace("Message", "");
			boolean found = false;

			for (Method method : this.getClass().getMethods()) {
				String methName = method.getName();

				if (methName.equalsIgnoreCase(className) || methName.equalsIgnoreCase("on" + className)) {
					_methodCache.put(m.getClass(), method);
					found = true;
					break;
				}
			}

			if (!found) {
				_methodCache.put(m.getClass(), null);
			}
		}

		Method method = _methodCache.get(m.getClass());
		Object[] args;

		if (method == null) {
			return;
		}

		if (method.getParameterTypes().length == 1) {
			args = new Object[]{_game};
		} else {
			args = new Object[]{_game, m};
		}

		try {
			method.setAccessible(true);
			method.invoke(this, args);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			e.printStackTrace();
		}

		// TODO: move that elsewhere?
		_game.repaint();
	}
}
