package fr.ensicaen.ecole.easygame.state;

import java.awt.Graphics;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Implements the game state, i.e. all the information about the game being played.
 */
public class GameState implements Serializable {

	private static final long serialVersionUID = 1L;

	private Board _board;
	private List<Player> _players = new ArrayList<>();
	private PlayerOrder _order = new AscendingPlayerOrder();
	private Player _currentPlayer;
	private boolean _isEndOfGame;
	private boolean _displayAllInventory = false;

	/**
	 * Creates a game state.
	 *
	 * @param board Game board associated to the game.
	 */
	public GameState(Board board) {
		_board = board;
	}

	/**
	 * Creates a game state with a specific player order.
	 *
	 * @param board Game board associated to the game.
	 * @param order Order in which players play.
	 */
	public GameState(Board board, PlayerOrder order) {
		_board = board;
		_order = order;
	}

	/**
	 * Adds a player to the game.
	 *
	 * @param p Player to be added.
	 */
	public final void addPlayer(Player p) {
		_players.add(p);
	}

	/**
	 * Goes to the next turn.
	 */
	public final void nextTurn() {
		_currentPlayer = _order.getNextPlayer(this, _currentPlayer);
	}

	/**
	 * Draw the board.
	 *
	 * @param g Graphics context used for rendering.
	 */
	public final void draw(Graphics g) {
		_board.draw(g);
	}

	/**
	 * @return The game board.
	 */
	public final Board getBoard() {
		return _board;
	}

	/**
	 * @return The player currently playing.
	 */
	public final Player getCurrentPlayer() {
		return _currentPlayer;
	}

	/**
	 * @return The list of all players in the game.
	 */
	public final List<Player> getPlayers() {
		return _players;
	}

	/**
	 * Ends the game.
	 */
	public final void setEndOfGame() {
		_isEndOfGame = true;
	}

	/**
	 * @return Whether or not the game has ended.
	 */
	public final boolean isEndOfGame() {
		return _isEndOfGame;
	}

	/**
	 * Sets whether or not every inventory can be seen by any player.
	 *
	 * @param b true if all inventories are public. false otherwise.
	 */
	public final void setDisplayAllInventory(boolean b) {
		_displayAllInventory = b;
	}

	/**
	 * @return Whether or not every inventory can be seen by any player.
	 */
	public final boolean getDisplayAllInventory() {
		return _displayAllInventory;
	}

	/**
	 * Sets the order in which the player play.
	 *
	 * @param order Implementation of the order.
	 */
	public final void setPlayerOrder(PlayerOrder order) {
		this._order = order;
	}
}
