package fr.ensicaen.ecole.easygame.state;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JList;

import fr.ensicaen.ecole.easygame.gui.EntityListModel;
import fr.ensicaen.ecole.easygame.gui.InventoryZone;

/**
 * A player's inventory i.e. the entities he owns and that are not on the board.
 */
public final class Inventory implements Serializable {

	private static final long serialVersionUID = 1L;

	private transient InventoryZone _gui;
	private Player _player;
	private final List<Entity> _entities = new ArrayList<Entity>();

	public Inventory(Player p) {
		_player = p;
		_gui = null;
	}

	/**
	 * Adds an entity to the inventory. Also adds it to the player's list of owned entities.
	 *
	 * @param e Entity to be added.
	 */
	public void addEntity(Entity e) {
		_entities.add(e);
		e.setPlayer(_player);

		if (_gui != null) {
			_gui.updateInventoryView();
		}
	}

	/**
	 * Removes an entity from a player's inventory.
	 * Warning: The entity is not removed from the player's list of owned entities.
	 *
	 * @param e The entity to be removed from the inventory.
	 */
	public void removeEntity(Entity e) {
		_entities.remove(e);

		if (_gui != null) {
			_gui.updateInventoryView();
		}
	}

	public void setGui(InventoryZone gui) {
		this._gui = gui;
	}

	/**
	 * @return Whether or not this player's inventory is the one currently displayed in the inventory window.
	 */
	public boolean isCurrentlyDisplayed() {
		return _gui != null && _gui.getSelectedPlayer() == _player.getId();
	}

	/**
	 * @return The entity currently selected. null if none is selected.
	 */
	public Entity getSelectedEntity() {
		if (!isCurrentlyDisplayed()) {
			return null;
		}

		return _gui.getSelectedEntity();
	}

	/**
	 * Updates the inventory display.
	 */
	public void updateView() {
		if (_gui != null) {
			_gui.updateInventoryView();
		}
	}

	/**
	 * @return Whether the inventory is empty or not.
	 */
	public boolean isEmpty() {
		return _entities.isEmpty();
	}

	/**
	 * @return The number of entities in the inventory.
	 */
	public int getEntityCount() {
		return _entities.size();
	}

	/**
	 * Warning: no existence check is done.
	 *
	 * @param i Index of the entity to be returned.
	 * @return The entity at the given index i.
	 */
	public Entity getEntity(int i) {
		return _entities.get(i);
	}

	public void updateList(JList<Entity> list, EntityListModel model) {
		synchronized (_entities) {
			int count = getEntityCount();
			int j = 0;

			for (int i = 0; i < count; ++i) {
				if (getEntity(i) == null) {
					count--;
				}
			}

			Entity[] array = new Entity[count];

			for (int i = 0; i < count; ++i) {
				if (getEntity(i) != null) {
					array[j++] = getEntity(i);
				}
			}

			list.setListData(array);
			model.setList(_entities);
		}
	}
}
