package fr.ensicaen.ecole.easygame.state;

import java.awt.Graphics;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import fr.ensicaen.ecole.easygame.messaging.AddEntityToZoneMessage;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;

/**
 * A zone displayed on the game board containing entities.
 */
public class Zone implements Drawable, Serializable {

	private static final long serialVersionUID = 1L;

	private List<Entity> _entities = new ArrayList<Entity>();
	private int _x;
	private int _y;
	private Shape _shape;

	/**
	 * Constructs a zone.
	 *
	 * @param x x-coordinate of the zone.
	 * @param y y-coordinate of the zone.
	 * @param shape Shape of the zone.
	 */
	public Zone(int x, int y, Shape shape) {
		_x = x;
		_y = y;
		_shape = shape;
	}

	/**
	 * Adds an entity to the zone.
	 * Triggers an AddEntityToZoneMessage.
	 *
	 * @param e Entity to be added.
	 */
	public void addEntity(Entity e) {
		_entities.add(e);
		MessageReceiver.receive(new AddEntityToZoneMessage(this, e));
	}

	/**
	 * Removes an entity from the zone.
	 *
	 * @param e Entity to be removed.
	 */
	public void removeEntity(Entity e) {
		_entities.remove(e);
	}

	/**
	 * Removes all entities from the zone.
	 */
	public void clear() {
		_entities.clear();
	}

	/**
	 * @return Whether the zone is empty or not.
	 */
	public boolean isEmpty() {
		return _entities.isEmpty();
	}

	public void draw(Graphics g) {
		g.translate(_x, _y);
		_shape.draw(g);

		for (Entity e : _entities) {
			e.draw(g);
		}

		g.translate(-_x, -_y);
	}

	/**
	 * Tests if a click is in the zone.
	 *
	 * @param x x-coordinate of the click.
	 * @param y y-coordinate of the click.
	 * @return Whether the click is in the zone or not.
	 */
	public boolean contains(int x, int y) {
		return _shape.contains(x - _x, y - _y);
	}

	/**
	 * @return The shape of the zone.
	 */
	public Shape getShape() {
		return _shape;
	}

	/**
	 * @param i Index of the entity to return.
	 * @return The entity of the given i index.
	 */
	public Entity getEntity(int i) {
		return _entities.get(i);
	}

	/**
	 * @return x-coordinate of the zone.
	 */
	public int getX() {
		return _x;
	}

	/**
	 * @return y-coordinate of the zone.
	 */
	public int getY() {
		return _y;
	}

	/**
	 * @return Number of entities in the zone.
	 */
	public int getNbEntities() {
		return _entities.size();
	}
}
