package net.kidanger.yourchess.yourchess;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import net.kidanger.yourchess.yourchess.coup.Coup;
import net.kidanger.yourchess.yourchess.coup.Roque;
import net.kidanger.yourchess.yourchess.piece.Cavalier;
import net.kidanger.yourchess.yourchess.piece.Dame;
import net.kidanger.yourchess.yourchess.piece.Fou;
import net.kidanger.yourchess.yourchess.piece.Piece;
import net.kidanger.yourchess.yourchess.piece.Pion;
import net.kidanger.yourchess.yourchess.piece.Roi;
import net.kidanger.yourchess.yourchess.piece.Tour;

public class Echiquier implements Serializable {

	// Optimisation : au lieu de créer une liste avec toutes les pièces, créé deux listes,
	// en fonction de la couleur de la pièce. Cela permet à getPieces(Couleur) d'être beaucoup plus rapide.
	private HashMap<Couleur, List<Piece>> pieces = new HashMap<>();

	// Optimisation : pour getPiecePosition, permet d'acceder en temps constant à la pièce recherchée.
	private Piece[] piecesT = new Piece[64];

	// Optimisation : permet de convertir un Echiquier en long.
	// Cela permet de stocker des échiquier dans des hashmap, pour mémoriser ceux correspondent à un echec par exemple.
	private static long[][] keys = new long[12][64]; // Zobrist hashing
	private long hash = 0;

	// Optimisation : puisque la méthode getRoi(Couleur) est souvent appellée, cela permet de ne pas avoir à faire une recherche complète à chaque appel.
	private HashMap<Couleur, Roi> rois = new HashMap<>();

	public Echiquier() {
		pieces.put(Couleur.blanc, new LinkedList<Piece>());
		pieces.put(Couleur.noir, new LinkedList<Piece>());

		// Initialise le tableau qui permet de hasher l'échiquier
		Random r = new Random(500);
		for (int i = 0; i < 12; i++) {
			for (int j = 0; j < 64; j++) {
				keys[i][j] = r.nextLong();
			}
		}
	}

	public Piece getPiecePosition(Position p) {
		return getPiecePosition(p.getX(), p.getY());
	}

	public Piece getPiecePosition(int x, int y) {
		return piecesT[(x - 1) * 8 + (y - 1)];
	}

	public List<Piece> getPieces(Couleur c) {
		return pieces.get(c);
	}

	public List<Piece> getPieces() {
		List<Piece> liste = new LinkedList<>();
		for (List<Piece> l : pieces.values()) {
			liste.addAll(l);
		}
		return liste;
	}

	/**
	 * Met à jour de hash. Si la piece n'était pas présente, elle est ajoutée. Si elle était présente, elle est enlevée. (Principe de XOR)
	 */
	private void updatePiece(Piece p) {
		hash ^= keys[pieceHash(p) - 1][p.getPosition().hashCode()];
	}

	/**
	 * Appelée lors qu'une pièce change de position. Met à jour le tableau.
	 */
	private void setAt(Position p, Piece piece) {
		piecesT[p.hashCode()] = piece;
	}

	public void remove(Piece p) {
		updatePiece(p);
		setAt(p.getPosition(), null);
		pieces.get(p.getCouleur()).remove(p);
		p.setAlive(false);
	}

	private int pieceHash(Piece p) {
		int h = 0;
		if (p instanceof Pion) {
			h = 1;
		} else if (p instanceof Cavalier) {
			h = 2;
		} else if (p instanceof Tour) {
			h = 3;
		} else if (p instanceof Dame) {
			h = 4;
		} else if (p instanceof Fou) {
			h = 5;
		} else if (p instanceof Roi) {
			h = 6;
		}
		if (p.getCouleur().isNoir())
			h += 6;
		return h;
	}

	public void add(Piece p) {
		getPieces(p.getCouleur()).add(p);
		setAt(p.getPosition(), p);

		p.setAlive(true);
		updatePiece(p); // met à jour le hashcode

		Collections.sort(getPieces(p.getCouleur()));

		if (p.estRoi()) {
			rois.put(p.getCouleur(), (Roi) p);
		}
	}

	/**
	 * Déplace une pièce. Appelle sa méthode onMove et met à jour l'échiquier.
	 */
	public void move(Piece p, Position pos) {
		updatePiece(p);
		setAt(p.getPosition(), null);

		p.onMove(pos);

		updatePiece(p);
		setAt(p.getPosition(), p);
	}

	/**
	 * Comme move, mais appel onUnmove
	 */
	public void unmove(Piece p, Position pos) {
		updatePiece(p);
		setAt(p.getPosition(), null); // supprime la pièce.

		p.onUnmove(pos);

		updatePiece(p);
		setAt(p.getPosition(), p);
	}

	/**
	 * Déplace les pièces en fonction du type de roque et de la couleur.
	 */
	public void realiserRoque(Roque coup) {
		int y = coup.getPiece().getPosition().getY();
		if (coup.isPetitRoque()) {
			Piece tour = getPiecePosition(8, y);
			move(tour, Position.get(6, y));
		} else {
			Piece tour = getPiecePosition(1, y);
			move(tour, Position.get(4, y));
		}
		move(coup.getPiece(), coup.getDestination());
	}

	/**
	 * Annule un roque.
	 */
	public void derealiserRoque(Roque coup) {
		int y = coup.getPiece().getPosition().getY();
		if (coup.isPetitRoque()) {
			Piece tour = getPiecePosition(6, y);
			unmove(tour, Position.get(8, y));
		} else {
			Piece tour = getPiecePosition(4, y);
			unmove(tour, Position.get(1, y));
		}
		unmove(coup.getPiece(), coup.getSource()); // le roi
	}

	/**
	 * Remplace une piece par une autre. Utilisé par la classe Promotion.
	 */
	public void promotionPiece(Piece origine, Piece nouvelle) {
		remove(origine);
		origine.setDisplay(false); // rend la piece invible

		add(nouvelle);
		nouvelle.setDisplay(true);
	}

	/**
	 * Renvoie le roi de la couleur passée en paramètre.
	 */
	public Roi getRoi(Couleur c) {
		return rois.get(c);
	}

	/**
	 * Joue un coup.
	 */
	public void jouerCoup(Couleur c, Coup coup) {
		coup.jouer(this, c);
		hash ^= 1; // bascule le bit correspond au joueur actuel.
	}

	public void dejouerCoup(Couleur c, Coup coup) {
		coup.dejouer(this, c);
		hash ^= 1;
	}

	/**
	 * Renvoie le hashcode de l'échiquier. Elle renvoie un long car un int ne permet pas de stocker toutes les combinaisons sans collision.
	 */
	public long hashCode2() {
		return hash;
	}

	public String toString() {
		return "" + hash;
	}
}
