package net.kidanger.yourchess.yourchess;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;

import net.kidanger.yourchess.yourchess.coup.Coup;
import net.kidanger.yourchess.yourchess.coup.CoupParser;
import net.kidanger.yourchess.yourchess.coup.Deplacement;
import net.kidanger.yourchess.yourchess.coup.IDeplacement;
import net.kidanger.yourchess.yourchess.coup.IPrise;
import net.kidanger.yourchess.yourchess.coup.PriseEnPassant;
import net.kidanger.yourchess.yourchess.coup.Promotion;
import net.kidanger.yourchess.yourchess.coup.Roque;
import net.kidanger.yourchess.yourchess.ordinateur.OrdinateurAleatoire;
import net.kidanger.yourchess.yourchess.ordinateur.OrdinateurIntelligent;
import net.kidanger.yourchess.yourchess.ordinateur.OrdinateurMoyen;
import net.kidanger.yourchess.yourchess.piece.Cavalier;
import net.kidanger.yourchess.yourchess.piece.Dame;
import net.kidanger.yourchess.yourchess.piece.Fou;
import net.kidanger.yourchess.yourchess.piece.Piece;
import net.kidanger.yourchess.yourchess.piece.Pion;
import net.kidanger.yourchess.yourchess.piece.Roi;
import net.kidanger.yourchess.yourchess.piece.Tour;

public class Partie extends Observable implements Serializable {

	private Echiquier echiquier = new Echiquier();
	private Joueur joueur1;
	private Joueur joueur2;
	private Joueur joueurActuel = null;
	private Joueur gagnant = null;

	private List<Coup> historiqueCoup = new ArrayList<>();
	private int historiqueIndex = 0; // pointe vers la prochaine case vide
	private List<Long> positions = new ArrayList<>(); // contient les différents échiquier depuis le début de la partie, permet de savoir les 3 positions identiques (estPartieNulle)

	private List<Coup> coupsNoir = null; // mémorise les coups jouable (court terme)
	private List<Coup> coupsBlanc = null;

	private static final int PREPARE = 0; // sélection des joueurs
	private static final int DEMARRE = 1; // prêt à être démarrée
	private static final int JOUE = 2; // en cours de jeu
	private static final int FIN = 4; // partie finie
	private int state = PREPARE;
	private String motifDeFin = null;

	public boolean estFinie() {
		return state == FIN;
	}

	private boolean peutAnnulerDernierCoup() {
		return state == JOUE && historiqueIndex != 0;
	}

	private boolean peutRejouerCoup() {
		return state == JOUE && historiqueIndex < historiqueCoup.size();
	}

	public Coup coupPrecedent() {
		if (!peutAnnulerDernierCoup())
			return null;
		return historiqueCoup.get(historiqueIndex - 1);
	}

	/*
	 * La méthode annulerDernierCoup permet d’annuler le dernier coup joué par un joueur, que ce joueur soit un humain ou un ordinateur.
	 */
	public void annulerDernierCoup() {
		if (!peutAnnulerDernierCoup()) {
			return;
		}
		Coup coup = coupPrecedent();
		historiqueIndex--;
		testDejouerCoup(joueurActuel.getCouleur().other(), coup);
		notifyObservers("annule");
		notifyObservers(echiquier);
		notifyObservers(joueurActuel);
	}

	/*
	 * La méthode rejouerCoupAnnule permet de rejouer le dernier coup annulé par la méthode annulerDernierCoup. Si plusieurs coups ont été annulés, le fait de répéter l’appel à cette méthode doit permettre de tous les rejouer.
	 */
	public void rejouerCoupAnnule() {
		if (!peutRejouerCoup()) {
			return;
		}
		Coup coup = historiqueCoup.get(historiqueIndex);
		historiqueIndex++;
		testJouerCoup(joueurActuel.getCouleur(), coup);
		notifyObservers("rejoue");
		notifyObservers(echiquier);
		notifyObservers(joueurActuel);
	}

	private void sauvegarderCoup(Coup coup) {
		// suppression des coups annulés
		while (historiqueCoup.size() > historiqueIndex) {
			historiqueCoup.remove(historiqueCoup.size() - 1);
		}
		historiqueCoup.add(historiqueIndex, coup);
		historiqueIndex++;
	}

	public List<Coup> getHistorique() {
		return historiqueCoup;
	}

	public int getHistoriqueIndex() {
		return historiqueIndex;
	}

	/*
	 * La méthode estEchec permet de déterminer si la configuration actuelle de la partie correspond à un échec au roi.
	 */
	public boolean estEchec(Joueur j) {
		return estEchec(j.getCouleur());
	}

	private boolean estEchec(Couleur c) {
		// on vérifie la situation
		Piece roi = echiquier.getRoi(c);
		Couleur ennemi = c.other();

		// pour chacune des pièces,
		for (Piece p : echiquier.getPieces(ennemi)) {
			// pour chaque position qu'elle peut atteindre
			for (Position pos : p.positionsAtteignableEchiquier(echiquier)) {
				// si le roi est à la position atteignable
				if (pos.equals(roi.getPosition())) {
					return true;
				}
			}
		}
		return false;
	}

	/*
	 * La méthode estPat permet de déterminer si la configuration de la partie correspond à un pat du roi.
	 */
	private boolean estPat(Couleur c) {
		if (estEchec(c)) {
			return false;
		}
		if (joueurActuel.getCouleur() != c) {
			return false;
		}
		// Si le seul coup jouable est de mettre son roi en échec (ce qui est
		// interdit) le roi est dit pat et la partie est nulle.
		for (Coup coup : listeDesCoupsCouleur(c)) {
			testJouerCoup(c, coup);
			if (!estEchec(c)) {
				testDejouerCoup(c, coup);
				return false;
			}
			testDejouerCoup(c, coup);
		}
		return true;
	}

	/*
	 * La méthode estEchecEtMat permet de vérifier que la configuration actuelle des pièces correspond à un échec et mat (donc à la fin de la partie).
	 */
	public boolean estEchecEtMat(Couleur c) {
		if (!estEchec(c)) {
			return false;
		}
		if (joueurActuel.getCouleur() != c) {
			return false;
		}
		// L'expression "Echec et mat" signifie que le roi adverse reçoit un
		// échec imparable.
		for (Coup coup : listeDesCoupsCouleur(c)) {
			testJouerCoup(c, coup);
			if (!estEchec(c)) {
				testDejouerCoup(c, coup);
				return false;
			}
			testDejouerCoup(c, coup);
		}
		return true;
	}

	/*
	 * La méthode estPartieNulle permet de vérifier le déroulement de la partie a conduit à une partie nulle (donc à la fin de la partie). Le pat fait partie de ce cas, mais ce n’est pas le seul cas (http://iechecs.com/regles.htm#nulle), comme le fait que pendant 50 coups, il n’y ait aucune prise ou mouvement de pions.
	 */
	public boolean estPartieNulle() {
		if (state == FIN) {
			if (gagnant != null) {
				motifDeFin = "Gagnant";
				return false;
			}
			return true;
		}

		if (estPat(Couleur.blanc)) {
			motifDeFin = "Blancs pat";
			return true;
		}
		if (estPat(Couleur.noir)) {
			motifDeFin = "Noirs pat";
			return true;
		}

		if (listeDesCoupsCouleur(Couleur.blanc).isEmpty()) {
			motifDeFin = "Coup blancs vide";
			return true;
		}
		if (listeDesCoupsCouleur(Couleur.noir).isEmpty()) {
			motifDeFin = "Coup noirs vide";
			return true;
		}

		// Test du matériel : R - R, RF - R, RF - RF, et RC - R
		if (echiquier.getPieces(Couleur.noir).size() == 1 && echiquier.getPieces(Couleur.blanc).size() == 1) {
			motifDeFin = "R vs R";
			return true;
		}
		boolean foublanc = false;
		boolean founoir = false;
		boolean cavalierblanc = false;
		boolean cavaliernoir = false;
		boolean otherblanc = false;
		boolean othernoir = false;
		for (Piece p : echiquier.getPieces(Couleur.blanc)) {
			if (p instanceof Roi) {
			} else if (p instanceof Fou && !foublanc) {
				foublanc = true;
			} else if (p instanceof Cavalier && !cavalierblanc) {
				cavalierblanc = true;
			} else {
				otherblanc = true;
			}
		}
		for (Piece p : echiquier.getPieces(Couleur.noir)) {
			if (p instanceof Roi) {
			} else if (p instanceof Fou && !founoir) {
				founoir = true;
			} else if (p instanceof Cavalier && !cavaliernoir) {
				cavaliernoir = true;
			} else {
				othernoir = true;
			}
		}
		if (!otherblanc && !othernoir) {
			motifDeFin = "Impossible de mater";
			return true;
		}

		// Recherche de trois autres positions similaires
		int n = 0;
		for (long position : positions) {
			if (position == echiquier.hashCode2())
				n++;
			if (n == 3)
				break;
		}
		if (n == 3) {
			motifDeFin = "Trois positions identiques";
			return true;
		}

		// 50 coups sans prise ni déplacement de pion
		if (historiqueIndex > 50) {
			boolean pasDePrise = true;
			for (int h = historiqueIndex - 1; h >= 0; h--) {
				Coup c = historiqueCoup.get(h);
				if (c instanceof IDeplacement) {
					IDeplacement cd = (IDeplacement) c;
					if (cd.getPiece() instanceof Pion) {
						pasDePrise = false;
						break;
					}
				}
				if (c instanceof IPrise) {
					IPrise cp = (IPrise) c;
					if (cp.getPrise() != null) {
						pasDePrise = false;
						break;
					}
				}
			}

			if (pasDePrise) {
				motifDeFin = "Pas de prise ni déplacement de pion depuis 50 coups";
				return true;
			}
		}

		motifDeFin = null;
		return false;
	}

	public void testJouerCoup(Couleur c, Coup coup) {
		echiquier.jouerCoup(c, coup);
		positions.add(echiquier.hashCode2());
		coupsBlanc = null;
		coupsNoir = null;

		joueurSuivant();
	}

	public void testDejouerCoup(Couleur c, Coup coup) {
		positions.remove(positions.size() - 1);
		coupsBlanc = null;
		coupsNoir = null;
		echiquier.dejouerCoup(c, coup);

		joueurSuivant();
	}

	/*
	 * Les méthodes sauvegardePartie et chargePartie permettent respectivement de sauver une partie, puis de la recharger à partir d’un fichier. Le format du fichier doit être ASCII afin de pouvoir comprendre les données enregistrées en l’ouvrant avec un éditeur de texte.
	 */
	public boolean chargePartie(String nom) {
		if (state != PREPARE) {
			return false;
		}
		String j1;
		String j2;
		BufferedReader r;
		try {
			r = new BufferedReader(new FileReader(nom));
			j1 = r.readLine().substring("joueur1:".length()).trim();
			j2 = r.readLine().substring("joueur2:".length()).trim();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		} catch (StringIndexOutOfBoundsException e) {
			e.printStackTrace();
			return false;
		}

		Joueur joueur1, joueur2;
		switch (j1) {
		case "ordinateur aléatoire":
			joueur1 = new OrdinateurAleatoire(Couleur.blanc);
			break;
		case "ordinateur moyen":
			joueur1 = new OrdinateurMoyen(Couleur.blanc);
			break;
		case "ordinateur intelligent":
			joueur1 = new OrdinateurIntelligent(Couleur.blanc);
			break;
		default:
			joueur1 = new JoueurHumain(j1, Couleur.blanc);
			break;
		}
		switch (j2) {
		case "ordinateur aléatoire":
			joueur2 = new OrdinateurAleatoire(Couleur.noir);
			break;
		case "ordinateur moyen":
			joueur2 = new OrdinateurMoyen(Couleur.noir);
			break;
		case "ordinateur intelligent":
			joueur2 = new OrdinateurIntelligent(Couleur.noir);
			break;
		default:
			joueur2 = new JoueurHumain(j2, Couleur.noir);
			break;
		}

		preparerPartie(joueur1, joueur2);

		// chargement de l'historique des coups
		String histo;
		try {
			histo = r.readLine().substring("historique:".length()).trim();
			r.close();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		Couleur c = Couleur.blanc;
		for (String s : histo.split(" ")) {
			Coup coup = CoupParser.parseComplet(echiquier, c, s);
			if (coup == null) {
				System.out.println("[chargement] Coup invalide : " + s);
				return false;
			}
			testJouerCoup(c, coup);
			sauvegarderCoup(coup);
			c = c.other();
		}

		// restoration de l'état de la partie
		verifierEtat();
		return true;
	}

	public boolean sauvegardePartie(String nom) {
		BufferedWriter w;
		try {
			w = new BufferedWriter(new FileWriter(nom));
			String s = null;
			if (joueur1 instanceof JoueurHumain)
				s = joueur1.getNom();
			else if (joueur1 instanceof OrdinateurIntelligent)
				s = "ordinateur intelligent";
			else if (joueur1 instanceof OrdinateurMoyen)
				s = "ordinateur moyen";
			else if (joueur1 instanceof OrdinateurAleatoire)
				s = "ordinateur aléatoire";
			w.write("joueur1: " + s);
			w.newLine();
			if (joueur2 instanceof JoueurHumain)
				s = joueur2.getNom();
			else if (joueur2 instanceof OrdinateurIntelligent)
				s = "ordinateur intelligent";
			else if (joueur2 instanceof OrdinateurMoyen)
				s = "ordinateur moyen";
			else if (joueur2 instanceof OrdinateurAleatoire)
				s = "ordinateur aléatoire";
			w.write("joueur2: " + s);
			w.newLine();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

		// sauvegarde de l'historique des coups
		StringBuilder histo = new StringBuilder("historique:");
		for (Coup coup : historiqueCoup) {
			String c = coup.notationComplete();
			histo.append(" ").append(c);
		}
		try {
			w.write(histo.toString());
			w.newLine();
			w.close();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	/*
	 * La méthode listeDesCoupsJouable retourne la liste de l’ensemble des mouvements valides pour la totalité des pièces jouables par un joueur donné.
	 */
	public List<Coup> listeDesCoupsCouleur(Couleur c) {
		if (joueur1.getCouleur() == c)
			return listeDesCoupsJouable(joueur1);
		return listeDesCoupsJouable(joueur2);
	}

	public List<Coup> listeDesCoupsJouable(Joueur j) {
		Couleur c = j.getCouleur();

		// si on l'a en cache, on la renvoie
		if (c.isBlanc() && coupsBlanc != null)
			return coupsBlanc;
		if (c.isNoir() && coupsNoir != null)
			return coupsNoir;

		List<Coup> liste = new LinkedList<>();
		// pour chaque piece de la couleur
		for (Piece p : echiquier.getPieces(c)) {
			List<Position> positions = p.positionsAtteignableEchiquier(echiquier);

			// pour chaque positions possibles, on ajoute le déplacement correspondant
			for (Position pos : positions) {
				Deplacement coup = null;
				Piece prise = echiquier.getPiecePosition(pos);
				if (prise == null) {
					if (p instanceof Pion && ((p.getCouleur().isBlanc() && pos.getY() == 8) || p.getCouleur().isNoir() && pos.getY() == 1))
						coup = new Promotion(p, pos);
					else
						coup = new Deplacement(p, pos);
				} else {
					if (!prise.estRoi()) {
						if (p.getCouleur() != prise.getCouleur() && p.getCouleur() == joueurActuel.getCouleur()) {
							if (p instanceof Pion && ((p.getCouleur().isBlanc() && pos.getY() == 8) || p.getCouleur().isNoir() && pos.getY() == 1))
								coup = new Promotion(p, prise);
							else
								coup = new Deplacement(p, prise);
						}
					}
				}
				if (coup != null) {
					liste.add(coup);
				}
			}

			// pour les pions
			if (p instanceof Pion) {
				// prises en passant :
				if (coupPrecedent() != null && coupPrecedent().isDeplacement()) {
					Deplacement prec = (Deplacement) coupPrecedent();
					if (prec.getPiece() instanceof Pion && Math.abs(prec.getDestination().getY() - prec.getSource().getY()) == 2) {
						int y = p.getPosition().getY();
						int x = p.getPosition().getX();
						if (p.getCouleur().isBlanc() && y == 5 || p.getCouleur().isNoir() && y == 4) {
							if (x < 8) {
								Piece target = echiquier.getPiecePosition(x + 1, y);
								if (target != null && prec.getPiece() == target) {
									liste.add(new PriseEnPassant((Pion) p, target));
								}
							}
							if (x > 1) {
								Piece target = echiquier.getPiecePosition(x - 1, y);
								if (target != null && prec.getPiece() == target) {
									liste.add(new PriseEnPassant((Pion) p, target));
								}
							}
						}
					}
				}
			}
		}

		// on ajoute les roques
		if (roqueValide(c, Roque.Type.PETIT)) {
			Coup coup = new Roque(echiquier.getRoi(c), Roque.Type.PETIT);
			liste.add(coup);
		}
		if (roqueValide(c, Roque.Type.GRAND)) {
			Coup coup = new Roque(echiquier.getRoi(c), Roque.Type.GRAND);
			liste.add(coup);
		}

		if (joueurActuel.getCouleur() != c) {
			return liste; // on peut pas tester les coups
		}

		// on retire les coups impossibles
		Iterator<Coup> it = liste.iterator();
		while (it.hasNext()) {
			Coup coup = it.next();
			boolean echec;
			testJouerCoup(c, coup);
			echec = estEchec(c);
			testDejouerCoup(c, coup);
			if (echec) {
				it.remove();
			}
		}

		if (c.isBlanc())
			coupsBlanc = liste;
		else
			coupsNoir = liste;
		return liste;
	}

	public List<Piece> getPiecesJoueur(Joueur j) {
		return echiquier.getPieces(j.getCouleur());
	}

	/*
	 * La méthode roqueValide retourne vrai, si les mouvements de pièces d’un joueur donné autorisent la réalisation du roque, ainsi que la configuration de l’échiquier. Il est nécessaire de préciser si le roque concerné est le petit ou le grand.
	 */
	private boolean roqueValide(Joueur j, Roque.Type type) {
		return roqueValide(j.getCouleur(), type);
	}

	private boolean roqueValide(Couleur c, Roque.Type type) {
		Roi r = echiquier.getRoi(c);
		if (r.hasMoved()) {
			return false;
		}
		int tour_y = c.isNoir() ? 8 : 1;
		int tour_x = 8;
		int dx = -1;
		if (type == Roque.Type.GRAND) {
			tour_x = 1;
			dx = 1;
		}
		Piece piece = echiquier.getPiecePosition(tour_x, tour_y);
		// si la pièce n'a pas bouger, c'est obligatoirement une tour
		if (piece == null || piece.hasMoved()) {
			return false;
		}
		int i = 1;
		while (tour_x + i * dx != r.getPosition().getX()) {
			int x = tour_x + i * dx;
			int y = tour_y;
			if (echiquier.getPiecePosition(x, y) != null) {
				return false;
			}
			for (Piece pe : echiquier.getPieces(c.other())) {
				for (Position pos : pe.positionsAtteignableEchiquier(echiquier))
					if (pos.equals(x, y))
						return false; // la case est contrôlée par l'ennemi
			}
			i++;
		}
		return true;
	}

	public void faireRoque(Joueur j, Roque.Type type) {
		if (roqueValide(j, type)) {
			jouerCoup(j, new Roque(echiquier.getRoi(j.getCouleur()), type));
		}
	}

	/*
	 * La méthode initialiserPositions place l’ensemble des pièces sur l’échiquier en tenant compte de leur couleur.
	 */
	private void initialiserPositions() {
		// Pièces blanches
		for (int x = 1; x <= 8; x++) {
			echiquier.add(new Pion(Position.get(x, 2), Couleur.blanc));
		}
		echiquier.add(new Tour(Position.get(1, 1), Couleur.blanc));
		echiquier.add(new Tour(Position.get(8, 1), Couleur.blanc));
		echiquier.add(new Cavalier(Position.get(2, 1), Couleur.blanc));
		echiquier.add(new Cavalier(Position.get(7, 1), Couleur.blanc));
		echiquier.add(new Fou(Position.get(3, 1), Couleur.blanc));
		echiquier.add(new Fou(Position.get(6, 1), Couleur.blanc));
		echiquier.add(new Dame(Position.get(4, 1), Couleur.blanc));
		echiquier.add(new Roi(Position.get(5, 1), Couleur.blanc));

		// Pièces noires
		for (int x = 1; x <= 8; x++) {
			echiquier.add(new Pion(Position.get(x, 7), Couleur.noir));
		}
		echiquier.add(new Tour(Position.get(1, 8), Couleur.noir));
		echiquier.add(new Tour(Position.get(8, 8), Couleur.noir));
		echiquier.add(new Cavalier(Position.get(2, 8), Couleur.noir));
		echiquier.add(new Cavalier(Position.get(7, 8), Couleur.noir));
		echiquier.add(new Fou(Position.get(3, 8), Couleur.noir));
		echiquier.add(new Fou(Position.get(6, 8), Couleur.noir));
		echiquier.add(new Dame(Position.get(4, 8), Couleur.noir));
		echiquier.add(new Roi(Position.get(5, 8), Couleur.noir));
	}

	public void preparerPartie(Joueur j1, Joueur j2) {
		if (state != PREPARE) {
			System.out.println("Mauvais état pour préparer");
			return;
		}
		if (j1 == null || j2 == null) {
			System.out.println(j1 + " " + j2 + " : pas valide pour préparer.");
			return;
		}
		if (j1.getCouleur() == j2.getCouleur()) {
			System.out.println(j1.getCouleur() + " : même couleur pour les 2 joueurs");
			return;
		}

		if (j1.getCouleur().isBlanc()) {
			this.joueur1 = j1;
			this.joueur2 = j2;
		} else {
			this.joueur1 = j2;
			this.joueur2 = j1;
		}
		j1.setPartie(this);
		j2.setPartie(this);
		initialiserPositions();
		joueurActuel = joueur1;
		state = DEMARRE;
		System.out.println("Partie démarrée");
	}

	/*
	 * La méthode commencerPartie lance le déroulement d’une partie en commençant par le premier joueur qui aura forcément les blancs.
	 */
	public void commencerPartie() {
		if (state != DEMARRE) {
			System.out.println("Mauvais état pour commencer: " + state);
			return;
		}

		state = JOUE;
		notifyObservers(joueurActuel);
		notifyObservers("joue");
	}

	public boolean hasDemarre() {
		return state > DEMARRE;
	}

	public boolean enJeu() {
		return state == JOUE;
	}

	public void nextStep() {
		if (state == JOUE) {
			Coup coup = joueurActuel.jouerCoup();
			if (coup != null) {
				jouerCoup(joueurActuel, coup);
			}
		} else if (state == FIN) {
			System.out.println("fini : " + joueurActuel);
		} else if (state == PREPARE) {
			System.out.println("Pas démarré");
		}
	}

	public synchronized boolean jouerCoup(Joueur j, Coup coup) {
		if (state != JOUE) {
			System.out.println("État invalide pour jouer un coup");
			return false;
		}

		if (j != joueurActuel) {
			System.out.println("pas à ton tour! j=" + j + ", joueurActuel=" + joueurActuel);
			return false;
		}

		boolean contains = false;
		for (Coup c : listeDesCoupsJouable(j)) {
			if (c.equals(coup)) {
				contains = true;
				break;
			}
		}
		if (!contains) {
			System.out.println("Pas dans la liste des coups !!");
			return false;
		}

		testJouerCoup(j.getCouleur(), coup);
		sauvegarderCoup(coup);

		notifyObservers(coup);

		// mise à jour de l'état de jeu
		verifierEtat();
		if (state == FIN) {
			notifyObservers("fini");
		} else {
			notifyObservers(joueurActuel);
		}
		notifyAll();
		System.out.println("coup joué " + coup);
		return true;
	}

	private void verifierEtat() {
		if (estEchecEtMat(Couleur.blanc)) {
			state = FIN;
			gagnant = joueur2;
		} else if (estEchecEtMat(Couleur.noir)) {
			state = FIN;
			gagnant = joueur1;
		} else if (estPartieNulle()) {
			state = FIN;
		}
	}

	public Joueur getGagnant() {
		return gagnant;
	}

	private void joueurSuivant() {
		if (joueurActuel == joueur1) {
			joueurActuel = joueur2;
		} else {
			joueurActuel = joueur1;
		}
	}

	public void notifyObservers(Object arg) {
		setChanged();
		super.notifyObservers(arg);
	}

	public Echiquier getEchiquier() {
		return echiquier;
	}

	public Joueur getJoueurActuel() {
		return joueurActuel;
	}

	public String getMotifDeFin() {
		if (!estPartieNulle())
			return null;
		return motifDeFin;
	}
}
